
#ifndef SVCMGR_H
#define SVCMGR_H

/*
C++ wrapper around Windows NT service control functions
Copyright (C) 2000 Antti Markus (antti@pld.ttu.ee)     
This code is based on Windows Foundation Classes by Samuel R. Blackburn
(http://ourworld.compuserve.com/homepages/sam_blackburn/wfc.htm)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string>
using namespace std;
#include <windows.h>

class TServiceStatus : public ENUM_SERVICE_STATUS
{ 
public:
  TServiceStatus();
  TServiceStatus(const ENUM_SERVICE_STATUS *pSource);
  ~TServiceStatus();                             
  void Empty(void);
  void Copy(const ENUM_SERVICE_STATUS *pSource); 
  string GetState(void);
};

class TServiceControlManager
{ 
private:
  // Don't allow canonical behavior (i.e. don't allow this class
  // to be passed by value)

  TServiceControlManager(const TServiceControlManager&) {};
  TServiceControlManager& operator=(const TServiceControlManager&) { return( *this ); }; 
  
  void Init(void);
protected: 
  SC_HANDLE hManagerHandle;
  ENUM_SERVICE_STATUS *pServiceStatusData;
  DWORD dwBufferSize;
  DWORD dwCurrentEntryNumber;
  DWORD dwErrorCode;
  DWORD dwNumberOfEntries;
  string strMachineName;
public:                 
  TServiceControlManager();
  virtual ~TServiceControlManager();
  void Close(void); 
  BOOL Open(DWORD dwAccess=SC_MANAGER_ALL_ACCESS,
            LPCTSTR strDatabase=NULL,
            LPCTSTR strMachine=NULL);     
  BOOL EnumerateStatus(DWORD dwState=SERVICE_ACTIVE,
                       DWORD dwType=SERVICE_WIN32);
                       
  //NB! this method allocates memory which has to be freed later!                      
  ENUM_SERVICE_STATUS *GetDependentServices(LPCTSTR pServiceName, DWORD &dwCount);                      
  BOOL GetNext(TServiceStatus &Status);
  DWORD GetServiceCount(void) const;  
  string GetMachine(void);
  SC_HANDLE GetHandle(void) const;                             
  DWORD GetErrorCode(void) const; 
  
  BOOL Start(LPCTSTR pServiceName, DWORD dwArgumentCount = 0, LPCTSTR *pArguments = NULL);
  BOOL Stop(LPCTSTR pServiceName);
};  

inline DWORD TServiceControlManager::GetServiceCount(void) const
{
  return dwNumberOfEntries;
}                         

inline string TServiceControlManager::GetMachine(void)
{                                     
  return strMachineName;
}                                 

inline SC_HANDLE TServiceControlManager::GetHandle(void) const
{
  return hManagerHandle;
}

inline DWORD TServiceControlManager::GetErrorCode(void) const
{                             
  return dwErrorCode;
}

#endif